package com.muyu.pay.wechat.clients.interceptor;

import com.dtflys.forest.http.ForestBody;
import com.dtflys.forest.http.ForestRequest;
import com.dtflys.forest.interceptor.Interceptor;
import com.dtflys.forest.reflection.ForestMethod;
import com.muyu.pay.wechat.config.WechatPayConfig;
import com.muyu.pay.wechat.domain.WeChatPayBaseReq;
import com.muyu.pay.wechat.domain.model.WeChatSignModel;
import com.muyu.pay.wechat.service.WeChatSignService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;

/**
 * @author DongZl
 * @description: 微信支付请求拦截器
 * @Date 2023-8-21 下午 03:06
 */
@Log4j2
@Component
public class WeChatAuthInterceptor<T> implements Interceptor<T> {

    @Autowired
    private WechatPayConfig wechatPayConfig;

    @Autowired
    private WeChatSignService weChatSignService;

    public WeChatAuthInterceptor () {
        log.info("微信支付请求拦截器 - 初始化成功");
    }

    /**
     * 该方法在被调用时，并在beforeExecute前被调用
     * @Param request Forest请求对象
     * @Param args 方法被调用时传入的参数数组
     */
    @Override
    public void onInvokeMethod(ForestRequest req, ForestMethod method, Object[] args) {
        log.debug("进行用户微信公众号和商户号填写");
        for (Object arg : args) {
            if (arg instanceof WeChatPayBaseReq){
                WeChatPayBaseReq weChatPayBaseReq = (WeChatPayBaseReq) arg;
                weChatPayBaseReq.setAppid(wechatPayConfig.getAppid());
                weChatPayBaseReq.setMchid(wechatPayConfig.getMerchantId());
            }
        }
    }

    /**
     * 该方法在请求发送之前被调用, 若返回false则不会继续发送请求
     * @Param request Forest请求对象
     */
    @Override
    public boolean beforeExecute(ForestRequest req) {
        log.debug("微信请求前拦截 - 添加Authorization授权请求头");
        ForestBody forestBody = req.getBody();
        String body = forestBody.encodeToString();
        String methodName = req.getType().getName();
        String url = req.getUrl();
        WeChatSignModel weChatSignModel = null;
        if (HttpMethod.GET.matches(methodName)){
            weChatSignModel = weChatSignService.tokenByGet(url);
        }else if (HttpMethod.POST.matches(methodName)){
            weChatSignModel = weChatSignService.tokenByPost(url, body);
        }
        // 添加Header
        req.addHeader("Authorization", weChatSignModel.getSign());
        // 继续执行请求返回true
        return true;
    }
}
