package com.muyu.pay.wechat.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;

import static com.muyu.pay.wechat.constant.WechatPayConstant.*;

/**
 * @author DongZl
 * @description: 微信支付配置
 * @Date 2023-8-21 下午 02:34
 */
@Configuration
public class WechatPayConfigure {

    public PrivateKey getPrivateKey(String filename) {

        try {
            String content = new String(Files.readAllBytes(Paths.get(filename)), StandardCharsets.UTF_8);
            String privateKey = content.replace("-----BEGIN PRIVATE KEY-----", "")
                    .replace("-----END PRIVATE KEY-----", "")
                    .replaceAll("\\s+", "");

            KeyFactory kf = KeyFactory.getInstance("RSA");
            return kf.generatePrivate(
                    new PKCS8EncodedKeySpec(Base64.getDecoder().decode(privateKey)));
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("当前Java环境不支持RSA", e);
        } catch (InvalidKeySpecException e) {
            throw new RuntimeException("无效的密钥格式");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 初始化微信支付配置
     * @return 微信支付配置
     */
    @Bean
    public WechatPayConfig initWechatPayConfig(){
        return WechatPayConfig.builder()
                .appid("*****")
                .merchantId("*****")
                .payKey("*****")
                .algorithmType(ALGORITHM_TYPE)
                .schema(SCHEMA)
                // 自己填我不知道
                .serialNo("******")
                // 自己填我不知道
                .privateKey(getPrivateKey("D:\\work\\demo\\wechat-pay\\src\\main\\resources\\wechat\\wxpaykey.pem"))
                .build();
    }
}
